#include <stdio.h>
#include <stdint.h>

// In C, right shift of a signed type does arithmetical shift (shifts in
// copies of the top bit), while right shift of an unsigned type does
// logical shift (shifts in zeros).  In two's-complement, -1 is all-ones,
// so arithmetical shift leaves it unchanged, but logical shift changes
// the top bit.
#define IS_SIGNED(type) (((type) -1) >> 1 == ((type) -1))

int main(void)
{
	/* why (int) ?  Because sizeof gives a size_t */
	printf("char           : %d\n", IS_SIGNED(char));
	printf("unsigned char  : %d\n", IS_SIGNED(unsigned char));
	printf("signed char    : %d\n", IS_SIGNED(signed char));
	printf("short          : %d\n", IS_SIGNED(short));
	printf("unsigned short : %d\n", IS_SIGNED(unsigned short));
	printf("int            : %d\n", IS_SIGNED(int));
	printf("unsigned       : %d\n", IS_SIGNED(unsigned));
	printf("long           : %d\n", IS_SIGNED(long));
	printf("unsigned long  : %d\n", IS_SIGNED(unsigned long));
	printf("long long      : %d\n", IS_SIGNED(long long));
	printf("u. long long   : %d\n", IS_SIGNED(unsigned long long));
	printf("uint32_t       : %d\n", IS_SIGNED(uint32_t));
	printf("int64_t        : %d\n", IS_SIGNED(int64_t));
	printf("size_t         : %d\n", IS_SIGNED(size_t));
	printf("intptr_t       : %d\n", IS_SIGNED(intptr_t));
	printf("intmax_t       : %d\n", IS_SIGNED(intmax_t));
	return 0;
}
